@@ -6,7 +6,7 @@
 !  Largo Pontecorvo 5                                         !
 !  56127 Pisa                                                 !
 !  Italy                                                      !
-!  Version 1.3 - March 2007                                   !
+!  Version 2.1 - June  2009                                   !
 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 !
 !
@@ -15,11 +15,10 @@ SUBROUTINE is(A, method, eps, maxit, G, drift, err)
   use smc_tools
   use smc_int
   use fi_int, only: gresidual
-  use is_int, only: drft_mg1
   USE f95_lapack
 !use la_precision
 use f77_lapack
-  use ponte_f_f, only: debug,verb,wout,fdrift
+!  use ponte_f_f, only: fdrift
   implicit none
   REAL(dp), DIMENSION(:,:,:):: a
   REAL(dp), DIMENSION(:,:) :: g
@@ -36,10 +35,14 @@ use f77_lapack
 
   REAL(dp)::alpha,gamma,check,binomial
   logical,external ::  sel
+! for debugging  lapack
+  character :: NORMIND="I"
+  REAL(dp) :: RCONDIZ
+!
  10 FORMAT("*")
  nverbose=1
 
-if(debug) write(*,*) "SUBROUTINE is: beginning"
+if(debug)  write(*,*) "SUBROUTINE is: beginning"
    n=size(a,1)
    nb=size(a,3)
    if (allocated(F)) deallocate (F)
@@ -63,7 +66,11 @@ nm=n*(nb-1)
   allocate(F(n*(nb-1),n*(nb-1)),H(n,n*nb),maux(0:nb-1,0:nb-1),s(n,n),&
         ipiv(n),ipivf(n*nb-n),x0(n),y0(n),x(n*(nb-1)),xi(n*(nb-1)),h0(n,n),&
         h1(n,n), Ftmp(n*(nb-1),n*(nb-1)),w(n),work(lwork),stat=info)
-   if (info/=0) return
+  if (info/=0) then
+        if(debug) write(*,*) "is: info=",info
+        info=1000
+        return
+  endif
 
 ! precompute coefficients
    do i=0,nb-1
@@ -112,8 +119,8 @@ nm=n*(nb-1)
    s=transpose(h(:,1:n))
    s(n,:)=1.d0
    call la_gesv(s,x0)
-   drift= drft_mg1(a)
-   fdrift=drift
+   drift= drftmg1(a)
+!   fdrift=drift
    gamma=sign(1.d0,drift)/dot_product(x0,matmul(h(:,n+1:2*n),y0))
    y0=y0*(1.d0*gamma) !!!!!!!!!!!
    do i=1,nb-2
@@ -131,7 +138,12 @@ nm=n*(nb-1)
    if(method/=2)then
       do iter=1,maxit
          ftmp=f
-         call la_getrf(f,ipivf)
+         call la_getrf(f,ipivf,RCONDIZ,NORMIND,INFO)
+         if (info/=0) then
+             if(debug) write(*,*) "is:la_getrf info=",info
+             info=2000
+             return
+         endif
          if(method==1)then
             alpha=1.d0
             do i=1,n
@@ -191,26 +203,27 @@ nm=n*(nb-1)
       ftmp=f
 
       ipivf=0
-      call DGEQP3( nm, nm, ftmp, nm, ipivf, x, WORK, LWORK, INFO )
-      if(info/=0)then
-        write(*,*)"error in la_geqp3 called by is"
-         write(wout,*)"error in la_geqp3 called by is"
-         return
+      call DGEQP3( nm, nm, ftmp, nm, ipivf, x, WORK, LWORK, info )
+      if (info/=0) then
+        if(debug) write(*,*) "is:la_dgeqp3 info=",info
+        info=2000
+        return
       endif
-     call DORGQR( nm, nm, nm, ftmp, nm, x, WORK, LWORK, INFO )
-      if(info/=0)then
-         write(wout,*)"error in la_geqp3 called by is"
-         return
+     call DORGQR( nm, nm, nm, ftmp, nm, x, WORK, LWORK, info )
+      if (info/=0) then
+        if(debug) write(*,*) "is:la_dgorgqr info=",info
+        info=2000
+        return
       endif
     else
 !   schur decomposition
       ftmp=f
       call la_gees(f,x,xi,ftmp,sel,sdim,info)
-     if(info/=0)then
-         write(wout,*)"error in la_gees called by is"
-         call print_it
-         return
-      end if
+      if (info/=0) then
+        if(debug) write(*,*) "is:la_gees info=",info
+        info=2000
+        return
+      endif
    end if
 
 ! compute G
@@ -219,51 +232,34 @@ nm=n*(nb-1)
   h0=h1-s
   CALL la_getrf(h0,ipiv)
   CALL la_getri(h0,ipiv,info)
-     if(info/=0)then
-         write(wout,*)"error in la_getri called by is"
-         call print_it
-         return
-      end if
+      if (info/=0) then
+        if(debug) write(*,*) "is:la_getri info=",info
+        info=2000
+        return
+      endif
   g=matmul(h1+s,h0)
   call gresidual(a,g,err)
  ! write(wout,*)"G-residual=",err
  ! call print_it
  ! write(wout,*)"drift=",drift
  ! call print_it
+  if (allocated(F)) deallocate (F)
+   if (allocated(H)) deallocate (H)
+   if (allocated(maux)) deallocate (maux)
+   if (allocated(s)) deallocate (s)
+   if (allocated(ipiv)) deallocate (ipiv)
+   if (allocated(ipivf)) deallocate (ipivf)
+   if (allocated(x0)) deallocate (x0)
+   if (allocated(y0)) deallocate (y0)
+  if (allocated(h0)) deallocate (h0)
+  if (allocated(h1)) deallocate (h1)
+   if (allocated(x)) deallocate (x)
+   if (allocated(xi)) deallocate (xi)
+   if (allocated(ftmp)) deallocate (ftmp)
+   if (allocated(w)) deallocate (w)
+   if (allocated(work)) deallocate (work)
 end SUBROUTINE is
 
-function drft_mg1(a)
-   use smc_tools
-  use smc_int, only: gth
-
-  implicit none
-  real(dp),dimension(:,:,:)::a
-  real(dp)::drft_mg1
-  !
-  real(dp),dimension(:,:),allocatable::s
-  real(dp),dimension(:),allocatable:: w,p
-  integer:: i,m,n
-   m=size(a,1)
-  n=size(a,3)
-
-
-  if (allocated(s)) deallocate(s)
-  if (allocated(p)) deallocate(p)
-    if (allocated(w)) deallocate(w)
-    allocate(s(m,m),p(m),w(m),stat=info)
-  if (info/=0) return
-  s=sum(a,dim=3)
-
-  call gth(s,w)
-  s=0.d0
-  do i=2,n
-     s=s+(i-1)*a(:,:,i)
-  end do
-  p=sum(s,dim=2)
-  drft_mg1=dot_product(p,w)-1.d0
-end function drft_mg1
-
-
 logical function sel(xr,xi)
 use smc_tools
 implicit none
