@@ -27,7 +27,7 @@
 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 !  Included in                                                !
 !  Structured Markov Chains Solver      [  SMCSolver  ]       !
-!  Version 1.3 - March 2007                                   !
+!  Version 2.1 - June  2009                                   !
 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
 
@@ -62,12 +62,12 @@ module pwcr_interface
      END SUBROUTINE residual
   endinterface
   INTERFACE 
-     SUBROUTINE spwcr(a,eps,g,err,maxit,intpmax)
+     SUBROUTINE spwcr(a,ds,eps,g,err,maxit,intpmax)
        IMPLICIT NONE
        REAL(KIND(0.d0)), DIMENSION(:,:,:) :: a
        REAL(KIND(0.d0)), DIMENSION(:,:)   :: g
        REAL(KIND(0.d0))                            :: err, eps
-       integer :: maxit,intpmax
+       integer :: maxit,intpmax,ds
      END SUBROUTINE spwcr
   endinterface
   INTERFACE
