@@ -78,7 +78,7 @@
 !=========================================================================!
 !  Included in                                                            !
 !  Structured Markov Chains Solver      [  SMCSolver  ]                   !
-!  Version 1.3 - March 2007                                               !
+!  Version 2.1 - June  2009                                               !
 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
 
@@ -86,9 +86,9 @@ SUBROUTINE PWCR(a,eps,g,err,maxit,intpmax)
   USE schur_interface
   USE pwcr_interface, ONLY : computeG, residual
   use smc_tools
-  use is_int, only: drft_mg1
+!  use smc_int, only: drftmg1
   use roots
-    use ponte_f_f, only: debug,verb,wout,fdrift
+!  use ponte_f_f, only: fdrift
   IMPLICIT NONE
   REAL(dp), DIMENSION(:,:,:):: a
   REAL(dp), DIMENSION(:,:) :: g
@@ -115,14 +115,18 @@ SUBROUTINE PWCR(a,eps,g,err,maxit,intpmax)
   !-------------------------------------
   m=SIZE(a,3)
   nb=SIZE(a,1)
-  if(allocated(ae)) deallocate(ae)
-  ALLOCATE(ae(nb,nb,m),stat=info)
-  if(info/=0)return
-  do j=1,m
-     ae(:,:,j)=transpose(a(:,:,j))
-  end do
-  fdrift= drft_mg1(ae)
-  deallocate(ae)
+!  if(allocated(ae)) deallocate(ae)
+!  ALLOCATE(ae(nb,nb,m),stat=info)
+!  if (info/=0) then
+!        if(debug) write(*,*) "pwcr: info=",info
+!        info=1000
+!        return
+!  endif
+!  do j=1,m
+!     ae(:,:,j)=transpose(a(:,:,j))
+!  end do
+!  fdrift= drftmg1(ae)
+!  deallocate(ae)
   l1=(m+1)/2
   n=l1
   ln=LOG(1.0d0*n)/LOG(2.0d0)
@@ -135,23 +139,55 @@ SUBROUTINE PWCR(a,eps,g,err,maxit,intpmax)
   if(allocated(a1)) deallocate(a1)
   if(allocated(a1s)) deallocate(a1s)
   ALLOCATE(ae(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "pwcr: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(ao(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "pwcr: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(hae(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "pwcr: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(hao(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "pwcr: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(a1(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "pwcr: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(a1s(nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "pwcr: info=",info
+        info=1000
+        return
+  endif
   if(allocated(mean)) deallocate(mean)
   allocate(mean(nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "pwcr: info=",info
+        info=1000
+        return
+  endif
   if(allocated(hmean)) deallocate(hmean)
   allocate(hmean(nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "pwcr: info=",info
+        info=1000
+        return
+  endif
 
   ae=0.0d0
   ao=0.0d0
@@ -171,7 +207,7 @@ SUBROUTINE PWCR(a,eps,g,err,maxit,intpmax)
      stoc=SUM(ao(:,l,:))+SUM(ae(:,l,:))
      IF(1.0d0-stoc>eps)THEN
 if(debug)  WRITE(*,*)"pwcr: The matrix A is not stochastic"
-        info=500
+        info=5000
         return
      ENDIF
   END DO
@@ -276,26 +312,32 @@ SUBROUTINE computeG(a1,a1s,ae,ao,hae,hao,eps,g)
      END DO loop10
      a1=TRANSPOSE(a1)
      CALL la_gesv(g, a1, info=info)
-     if (info/=0)then
-        write(*,*)"error in la_gesv with info=",info
+     if (info/=0) then
+        if(debug) write(*,*) "computeG:la_gesv info=",info
+        info=2000
         return
-     end if
+     endif
      g=TRANSPOSE(a1)
      RETURN
   ENDIF
   if (allocated(r)) deallocate(r)
   ALLOCATE(r(nb,nb),stat=info)
-if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "computeG: info=",info
+        info=1000
+        return
+  endif
   g=-TRANSPOSE(ao(:,:,1))
   loop100 : DO i=1,nb
      g(i,i)=g(i,i)+1.0d0
   END DO loop100
   r=TRANSPOSE(ae(:,:,1))
   CALL la_gesv(g, r,info=info)
-  if (info/=0)then
-     write(*,*)"error in la_gesv with info=",info
-     return
-  end if
+  if (info/=0) then
+        if(debug) write(*,*) "computeG:la_gesv info=",info
+        info=2000
+        return
+  endif
   r=TRANSPOSE(r)
   g=-hao(:,:,1)-MATMUL(hae(:,:,1),r)
   g=TRANSPOSE(g)
@@ -304,10 +346,11 @@ if(info/=0)return
   END DO loop200
   a1=TRANSPOSE(a1)
   CALL la_gesv(g, a1,info=info)  
-  if (info/=0)then
-     write(*,*)"error in la_gesv with info=",info
-     return
-  end if
+  if (info/=0) then
+        if(debug) write(*,*) "computeG:la_gesv info=",info
+        info=2000
+        return
+  endif
   g=TRANSPOSE(a1) 
 if(allocated(r))deallocate(r)
 END SUBROUTINE computeG
@@ -340,7 +383,11 @@ SUBROUTINE residual(a,g,err)
   n=SIZE(a,3)
   if(allocated(r)) deallocate(r)
   ALLOCATE(r(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "residual: info=",info
+        info=1000
+        return
+  endif
   r=a(:,:,n)
   loop1 : DO i=n-1,1,-1
      r=MATMUL(g,r)+a(:,:,i)
@@ -400,7 +447,6 @@ SUBROUTINE schur(ae,ao,hae,hao,mean,hmean,a1s,a0norm,eps,intpmaxl)
   USE fft_interface
   USE schur_interface, ONLY : sc1, sc2p, sc1p, sc2, scc2p, scc2, &
        means, test
- use ponte_f_f, only: debug,wout
   IMPLICIT NONE
   REAL(dp), allocatable, DIMENSION(:,:,:) :: ae, ao, hae, hao
   REAL(dp), DIMENSION(:)  :: mean, hmean, a1s
@@ -421,21 +467,53 @@ SUBROUTINE schur(ae,ao,hae,hao,mean,hmean,a1s,a0norm,eps,intpmaxl)
   if(allocated(thae)) deallocate(thae)
   if(allocated(thao)) deallocate(thao)
   ALLOCATE(tae(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "shur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(tao(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "shur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(a1(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "shur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(a2(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "shur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(a3(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "shur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(thae(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "shur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(thao(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "shur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(a4(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "shur: info=",info
+        info=1000
+        return
+  endif
 
   CALL ftb1(ae,tae)
   CALL ftb1(ao,tao)
@@ -458,29 +536,49 @@ SUBROUTINE schur(ae,ao,hae,hao,mean,hmean,a1s,a0norm,eps,intpmaxl)
         n=nn/2
         if(allocated(a1))deallocate(a1)
         allocate(a1(nb,nb,n),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+        endif
         a1=ae
         if (allocated(ae)) DEALLOCATE(ae)
         ALLOCATE(ae(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+        endif
         ae(:,:,1:n)=a1
         ae(:,:,n+1:nn)=0.0d0
         a1=ao
         if (allocated(ao)) DEALLOCATE(ao)
         ALLOCATE(ao(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+        endif
         ao(:,:,1:n)=a1
         ao(:,:,n+1:nn)=0.0d0
         a1=hae
         if (allocated(hae)) DEALLOCATE(hae)
         ALLOCATE(hae(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+        endif
         hae(:,:,1:n)=a1
         hae(:,:,n+1:nn)=0.0d0
         a1=hao
         if (allocated(hao)) DEALLOCATE(hao)
         ALLOCATE(hao(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+        endif
         hao(:,:,1:n)=a1
         hao(:,:,n+1:nn)=0.0d0
         if (allocated(tae)) deallocate(tae)
@@ -490,17 +588,41 @@ SUBROUTINE schur(ae,ao,hae,hao,mean,hmean,a1s,a0norm,eps,intpmaxl)
         if (allocated(a1)) deallocate(a1)
         if (allocated(a2)) deallocate(a2)
         allocate(tae(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+        endif
         allocate(tao(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+        endif
         allocate(thae(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+        endif
         allocate(thao(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+        endif
         allocate(a1(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+        endif
         allocate(a2(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+        endif
      ENDIF
      CALL ftb2(ae,tae)
      CALL ftb2(ao,tao)
@@ -526,13 +648,29 @@ SUBROUTINE schur(ae,ao,hae,hao,mean,hmean,a1s,a0norm,eps,intpmaxl)
   if (allocated(hao)) DEALLOCATE(hao)
   n=nn/2
   ALLOCATE(ae(nb,nb,n),stat=info)
-  if(info/=0) return
+  if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+  endif
   ALLOCATE(ao(nb,nb,n),stat=info)
-  if(info/=0) return
+  if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+  endif
   ALLOCATE(hae(nb,nb,n),stat=info)
-  if(info/=0) return
+  if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+  endif
   ALLOCATE(hao(nb,nb,n),stat=info)
-  if(info/=0) return
+  if (info/=0) then
+             if(debug) write(*,*) "shur: info=",info
+             info=1000
+             return
+  endif
   loop1 : DO l=1,nn/2
      ae(:,:,l)=a3(:,:,2*l-1)
      ao(:,:,l)=a3(:,:,2*l)
@@ -638,16 +776,32 @@ SUBROUTINE sc1p(tae,tao,a)
   nb=SIZE(tae,1)
   if(allocated(rmat))deallocate(rmat)
   ALLOCATE(rmat(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "scp1: info=",info
+             info=1000
+             return
+  endif
   if(allocated(termr))deallocate(termr)
   ALLOCATE(termr(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "scp1: info=",info
+             info=1000
+             return
+  endif
   if(allocated(cmat))deallocate(cmat)
   ALLOCATE(cmat(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "scp1: info=",info
+             info=1000
+             return
+  endif
   if(allocated(termc))deallocate(termc)
   ALLOCATE(termc(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "scp1: info=",info
+             info=1000
+             return
+  endif
   rmat(:,:)=-tao(:,:,1)
   loopi1 : DO i=1,nb
      rmat(i,i)=rmat(i,i)+1.0d0
@@ -724,10 +878,18 @@ SUBROUTINE sc2p(tae,tao,a1,a2)
   nb=SIZE(tae,1)
   if(allocated(rmat)) deallocate(rmat)
   ALLOCATE(rmat(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "scp2: info=",info
+             info=1000
+             return
+  endif
   if(allocated(cmat)) deallocate(cmat)
   ALLOCATE(cmat(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "scp2: info=",info
+             info=1000
+             return
+  endif
   rmat=MATMUL(a1(:,:,1),tae(:,:,1))
   a2(:,:,1)=tao(:,:,1)+rmat(:,:)
   nm1=n/2+1
@@ -788,10 +950,18 @@ SUBROUTINE scc2p(thae,thao,a1,a2)
   nb=SIZE(thae,1)
   if(allocated(rmat)) deallocate(rmat)
   ALLOCATE(rmat(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "scc2p: info=",info
+             info=1000
+             return
+  endif
   if(allocated(cmat)) deallocate(cmat)
   ALLOCATE(cmat(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "scc2p: info=",info
+             info=1000
+             return
+  endif
   rmat=MATMUL(a1(:,:,1),thae(:,:,1))
   a2(:,:,1)=thao(:,:,1)+rmat(:,:)
   nm1=n/2+1
@@ -843,10 +1013,18 @@ SUBROUTINE sc1(tae,tao,a)
   nb=SIZE(tae,1)
   if(allocated(cmat))deallocate(cmat)
   ALLOCATE(cmat(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "sc1: info=",info
+             info=1000
+             return
+  endif
   if(allocated(termc))deallocate(termc)
   ALLOCATE(termc(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "sc1: info=",info
+             info=1000
+             return
+  endif
   num=n/2
   loopl : DO l=1,num
      cmat=-tao(:,:,l)-(0.0d0,1.0d0)*tao(:,:,num+l)
@@ -898,10 +1076,18 @@ SUBROUTINE sc2(tae,tao,a1,a2)
   nb=SIZE(tae,1)
   if(allocated(cmat))deallocate(cmat)
   ALLOCATE(cmat(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "sc2: info=",info
+             info=1000
+             return
+  endif
   if(allocated(rmat))deallocate(rmat)
   ALLOCATE(rmat(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "sc2: info=",info
+             info=1000
+             return
+  endif
   num=n/2
   CALL fillroots(2*n)
   nmax=SIZE(wr,1)
@@ -952,10 +1138,18 @@ SUBROUTINE scc2(thae, thao, a1, a2)
   n=SIZE(thae,3)
   if(allocated(rmat))deallocate(rmat)
   ALLOCATE(rmat(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "scc2: info=",info
+             info=1000
+             return
+  endif
   if(allocated(cmat))deallocate(cmat)
   ALLOCATE(cmat(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "scc2: info=",info
+             info=1000
+             return
+  endif
   num=n/2
   loopl : DO l=1,num
      CALL prodc(a1,thae,l,num+l,rmat,cmat)
@@ -984,7 +1178,11 @@ SUBROUTINE prodc(a, b, ir, ic, rmat, cmat)
   nb=SIZE(a,1)
   if(allocated(aux))deallocate(aux)
   ALLOCATE(aux(nb,nb),stat=info)
- if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "prodc: info=",info
+             info=1000
+             return
+  endif
   aux(:,:)=a(:,:,ir)+a(:,:,ic)
   rmat(:,:)=b(:,:,ir)+b(:,:,ic)
   cmat=MATMUL(aux,rmat)
@@ -1036,10 +1234,18 @@ SUBROUTINE means(tae,tao,thae,mean,hmean)
   nb=SIZE(tae,1)
   if(allocated(w))deallocate(w)
   ALLOCATE(w(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "means: info=",info
+             info=1000
+             return
+  endif
   if(allocated(v))deallocate(v)
   ALLOCATE(v(nb,1),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+             if(debug) write(*,*) "means: info=",info
+             info=1000
+             return
+  endif
   oneh=1.d0/2.d0
   v(:,1)=1.d0-mean
   w(:,:)=-tao(:,:,1)
@@ -1128,11 +1334,12 @@ SUBROUTINE solver(rmat, termr)
   rmat=TRANSPOSE(rmat)
   IF(SIZE(termr,2)>1) termr=TRANSPOSE(termr)
   CALL la_gesv(rmat,termr,info=info)
+  if (info/=0) then
+        if(debug) write(*,*) "solver:la_gesv info=",info
+        info=2000
+        return
+  endif
   IF(SIZE(termr,2)>1) termr=TRANSPOSE(termr)
-  if (info/=0)then
-     write(*,*)"error in la_gesv with info=",info
-     return
-  end if
 END SUBROUTINE solver
 
 
@@ -1162,37 +1369,41 @@ SUBROUTINE solvec(cmat, termc)
   cmat=TRANSPOSE(cmat)
   termc=TRANSPOSE(termc)
   CALL la_gesv(cmat,termc,info=info)
+  if (info/=0) then
+        if(debug) write(*,*) "solvec:la_gesv info=",info
+        info=2000
+        return
+  endif
   termc=TRANSPOSE(termc)
-  if (info/=0)then
-     write(*,*)"error in la_gesv with info=",info
-     return
-  end if
 END SUBROUTINE solvec
 
-SUBROUTINE SPWCR(a,eps,g,err,maxit,intpmax)
+SUBROUTINE SPWCR(a,ds,eps,g,err,maxit,intpmax)
   USE schur_interface
   USE pwcr_interface, ONLY : scomputeG, residual
   use smc_tools
   use smc_int
   use roots
-  use ponte_f_f, only: debug,verb,wout,fdrift
+!  use ponte_f_f, only: fdrift
   IMPLICIT NONE
   REAL(dp), DIMENSION(:,:,:):: a
   REAL(dp), DIMENSION(:,:) :: g
   REAL(dp) :: eps,err
+  integer :: ds
 
   REAL(dp), DIMENSION(:,:,:), ALLOCATABLE :: ae, ao, hae, hao,sa
   REAL(dp), DIMENSION(:,:), ALLOCATABLE   :: a1,aa1
-  REAL(dp), DIMENSION(:), ALLOCATABLE     :: a1s
-  REAL(dp)                            :: stoc, a0norm,drift
+  REAL(dp), DIMENSION(:), ALLOCATABLE     :: a1s, uu,vv
+  REAL(dp)                            :: stoc, a0norm,drift,tau
   INTEGER                                     :: m, n, nb, j, &
-       l, l1, ln, step
-  integer :: maxit,intpmax, intpmaxl
+       l, l1, ln, step, i
+  integer :: maxit,intpmax, intpmaxl, dual=-1
+! dual va messo in input 
+
   intpmaxl=256*2**intpmax
 
 !20 FORMAT(1X,"*",\)
 20 FORMAT("*")
-if (debug) write(*,*) "spwcr:  Dimensions A =",size(a,1), size(a,2),size(a,3),&
+if (debug)  write(*,*) "spwcr:  Dimensions A =",size(a,1), size(a,2),size(a,3),&
 "   eps=",eps," err=",err," maxit=",maxit," intpmax=",intpmax
 
   !-------------------------------------
@@ -1202,10 +1413,29 @@ if (debug) write(*,*) "spwcr:  Dimensions A =",size(a,1), size(a,2),size(a,3),&
   nb=SIZE(a,1)
   if(allocated(aa1)) deallocate(aa1)
   ALLOCATE(aa1(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "spwcr: info=",info
+        info=1000
+        return
+  endif
+  if(allocated(uu)) deallocate(uu)
+  if(allocated(vv)) deallocate(vv)
+  ALLOCATE(uu(nb),vv(nb),stat=info)
+  if (info/=0) then
+        if(debug) write(*,*) "spwcr: info=",info
+        info=1000
+        return
+  endif
+
+
+
   if(allocated(sa)) deallocate(sa)
   ALLOCATE(sa(nb,nb,m),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "spwcr: info=",info
+        info=1000
+        return
+  endif
   sa=a
   do j=1,m
      sa(:,:,j)=transpose(sa(:,:,j))
@@ -1215,16 +1445,28 @@ if (debug) write(*,*) "spwcr:  Dimensions A =",size(a,1), size(a,2),size(a,3),&
      stoc=SUM(sa(l,:,:))
      IF(1.0d0-stoc>eps)THEN
 if(debug)  WRITE(*,*)"spwcr: The matrix A is not stochastic"
-        info=500
+        info=5000
         return
      ENDIF
   END DO
 
-  call shift(sa,aa1,drift)
-  fdrift=drift
+call mg_startup(sa,ds,dual,tau,uu,vv,drift)
+
+if (ds/=2) then
+   aa1=sa(:,:,1)
+end if
+!  call shift(sa,aa1,drift) 
+  call shift(sa,ds,tau,uu,vv,drift,sa)
+if(debug) write(*,*) "spwcr: after shift tau=",tau," drift=",drift," uu=",uu,"  vv=",vv," ds=",ds
+
+!  fdrift=drift
   do j=1,m
      sa(:,:,j)=transpose(sa(:,:,j))
   end do
+  if (ds==2) then
+     aa1=sa(:,:,1)
+  end if
+
 !  if(verb)then 
 !         write(wout,*)"drift=",drift
 !         call print_it
@@ -1246,17 +1488,41 @@ if(debug)  WRITE(*,*)"spwcr: The matrix A is not stochastic"
   if(allocated(a1)) deallocate(a1)
   if(allocated(a1s)) deallocate(a1s)
   ALLOCATE(ae(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "spwcr: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(ao(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "spwcr: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(hae(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "spwcr: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(hao(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "spwcr: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(a1(nb,nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "spwcr: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(a1s(nb),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "spwcr: info=",info
+        info=1000
+        return
+  endif
 
   ae=0.0d0
   ao=0.0d0
@@ -1274,11 +1540,13 @@ if(debug)  WRITE(*,*)"spwcr: The matrix A is not stochastic"
   ENDIF
   hae(:,:,1:n-1)=ae(:,:,2:n)
   hao=ao
-  if(drift>=0)then
+
+  if(ds==2)then
      a1=ae(:,:,1)
   else
      a1=transpose(aa1)
   end if
+
   !---------------------------------
   ! Start Pointwise Cyclic Reduction
   !---------------------------------
@@ -1303,6 +1571,14 @@ if(debug)  WRITE(*,*)"spwcr: The matrix A is not stochastic"
         endif
   END DO pwcrstage
   CALL scomputeG(a1,hao,g)
+  if(drift>0.and.ds==2) then
+     do i=1,nb
+        do j=1,nb
+           g(i,j)=g(i,j)+tau*uu(j)*vv(i)
+        end do
+     end do
+  end if
+
   CALL residual(a,g,err)
   g=transpose(g)
   if (allocated(ae)) deallocate(ae)
@@ -1316,6 +1592,8 @@ if(debug)  WRITE(*,*)"spwcr: The matrix A is not stochastic"
   if (allocated(wi)) deallocate(wi)
   if (allocated(wwr)) deallocate(wwr)
   if (allocated(wwi)) deallocate(wwi)
+  if(allocated(uu)) deallocate(uu)
+  if(allocated(vv)) deallocate(vv)
 END SUBROUTINE SPWCR
 
 
@@ -1334,18 +1612,21 @@ SUBROUTINE ScomputeG(a1,hao,g)
   END DO loop10
   a1=TRANSPOSE(a1)
   CALL la_gesv(g, a1, info=info)
-  if (info/=0)then
-     write(*,*)"ScomputeG: error in la_gesv with info=",info
-     return
-  end if
+  if (info/=0) then
+        if(debug) write(*,*) "scomputeG:la_gesv info=",info
+        info=2000
+        return
+  endif
   g=TRANSPOSE(a1)
 
 END SUBROUTINE scomputeG
+
+
+
 SUBROUTINE sschur(ae,ao,hae,hao,a0norm,eps,intpmaxl)
   use smc_tools
   USE fft_interface
   USE schur_interface, ONLY : sc1, sc2p, sc1p, sc2, scc2p, scc2,stest
- use ponte_f_f, only: debug,wout
   IMPLICIT NONE
   REAL(dp), allocatable, DIMENSION(:,:,:) :: ae, ao, hae, hao
 
@@ -1366,21 +1647,53 @@ SUBROUTINE sschur(ae,ao,hae,hao,a0norm,eps,intpmaxl)
   if(allocated(thae)) deallocate(thae)
   if(allocated(thao)) deallocate(thao)
   ALLOCATE(tae(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "sschur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(tao(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "sschur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(a1(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "sschur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(a2(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "sschur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(a3(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "sschur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(thae(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "sschur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(thao(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "sschur: info=",info
+        info=1000
+        return
+  endif
   ALLOCATE(a4(nb,nb,n),stat=info)
-  if(info/=0)return
+  if (info/=0) then
+        if(debug) write(*,*) "sschur: info=",info
+        info=1000
+        return
+  endif
 
   CALL ftb1(ae,tae)
   CALL ftb1(ao,tao)
@@ -1394,7 +1707,7 @@ SUBROUTINE sschur(ae,ao,hae,hao,a0norm,eps,intpmaxl)
   nn=n
   nradd=0
 100 nradd=nradd+1
- if (debug) write(*,*)"sschur: interp point",nn
+ if (debug)  write(*,*)"sschur: interp point",nn
   ! check the accuracy of the matrix coefficients
   CALL stest(a3,eps,answer)
   CALL stest(a4,eps,hanswer)
@@ -1403,29 +1716,49 @@ SUBROUTINE sschur(ae,ao,hae,hao,a0norm,eps,intpmaxl)
         n=nn/2
         if(allocated(a1))deallocate(a1)
         allocate(a1(nb,nb,n),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+        endif
         a1=ae
         if(allocated(ae)) DEALLOCATE(ae)
         ALLOCATE(ae(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+        endif
         ae(:,:,1:n)=a1
         ae(:,:,n+1:nn)=0.0d0
         a1=ao
         if (allocated(ao)) DEALLOCATE(ao)
         ALLOCATE(ao(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+        endif
         ao(:,:,1:n)=a1
         ao(:,:,n+1:nn)=0.0d0
         a1=hae
         if (allocated(hae)) DEALLOCATE(hae)
         ALLOCATE(hae(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+        endif
         hae(:,:,1:n)=a1
         hae(:,:,n+1:nn)=0.0d0
         a1=hao
         if (allocated(hao)) DEALLOCATE(hao)
         ALLOCATE(hao(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+        endif
         hao(:,:,1:n)=a1
         hao(:,:,n+1:nn)=0.0d0
         if (allocated(tae)) deallocate(tae)
@@ -1435,17 +1768,41 @@ SUBROUTINE sschur(ae,ao,hae,hao,a0norm,eps,intpmaxl)
         if (allocated(a1)) deallocate(a1)
         if (allocated(a2)) deallocate(a2)
         allocate(tae(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+        endif
         allocate(tao(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+        endif
         allocate(thae(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+        endif
         allocate(thao(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+        endif
         allocate(a1(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+        endif
         allocate(a2(nb,nb,nn),stat=info)
-        if(info/=0)return
+        if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+        endif
      ENDIF
      CALL ftb2(ae,tae)
      CALL ftb2(ao,tao)
@@ -1472,13 +1829,29 @@ SUBROUTINE sschur(ae,ao,hae,hao,a0norm,eps,intpmaxl)
   n=nn/2
 !   n=nn/4
   ALLOCATE(ae(nb,nb,n),stat=info)
-  if(info/=0) return
+  if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+  endif
   ALLOCATE(ao(nb,nb,n),stat=info)
-  if(info/=0) return
+  if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+  endif
   ALLOCATE(hae(nb,nb,n),stat=info)
-  if(info/=0) return
+  if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+  endif
   ALLOCATE(hao(nb,nb,n),stat=info)
-  if(info/=0) return
+  if (info/=0) then
+            if(debug) write(*,*) "sschur: info=",info
+            info=1000
+            return
+  endif
   loop1 : DO l=1,nn/2
 !  loop1 : DO l=1,nn/4
      ae(:,:,l)=a3(:,:,2*l-1)
@@ -1516,3 +1889,4 @@ SUBROUTINE stest(u, eps, answer)
   if(err>eps*n)answer=.true.
 end SUBROUTINE stest
 
+
