@@ -6,7 +6,7 @@
  * Largo Pontecorvo 5                                         *
  * 56127 Pisa                                                 *
  * Italy                                                      *
- * Version 1.3 - March 2007                                   *
+ * Version 2.1 - June  2009                                   *
  **************************************************************
  *
  *  callbacks.h
@@ -25,8 +25,6 @@ void on_main_menu_saver_activate (GtkMenuItem *, gpointer );
 void on_main_menu_saveu_activate (GtkMenuItem *, gpointer );
 void on_main_menu_savegru_activate (GtkMenuItem *, gpointer );
 void on_main_menu_savepi_activate (GtkMenuItem *, gpointer );
-void on_main_menu_imptel_activate (GtkMenuItem *, gpointer );
-void on_main_menu_exptel_activate (GtkMenuItem *, gpointer );
 void on_main_menu_savepad_activate (GtkMenuItem *, gpointer  );
 void on_main_menu_clearpad_activate(GtkMenuItem *, gpointer  );
 void on_main_menu_quit_activate    (GtkMenuItem *, gpointer  );
@@ -51,28 +49,20 @@ void on_main_menu_quit_leave       (GtkMenuItem *, gpointer  );
 void on_main_menu_problem_gm1      (GtkMenuItem *, gpointer );
 void on_main_menu_problem_mg1      (GtkMenuItem *, gpointer );
 void on_main_menu_problem_qbd      (GtkMenuItem *, gpointer );
-void on_main_menu_problem_examples_e    (GtkMenuItem *, gpointer );
-void on_main_menu_problem_examples_l    (GtkMenuItem *, gpointer );
-void on_main_menu_problem_example1    (GtkMenuItem *, gpointer );
-void on_main_menu_problem_example2    (GtkMenuItem *, gpointer );
-void on_main_menu_problem_example3    (GtkMenuItem *, gpointer );
-void on_main_menu_problem_example4    (GtkMenuItem *, gpointer );
-void on_main_menu_problem_example5    (GtkMenuItem *, gpointer );
-void on_main_menu_problem_example6    (GtkMenuItem *, gpointer );
-void on_main_menu_problem_example7    (GtkMenuItem *, gpointer );
-void on_main_menu_problem_example8    (GtkMenuItem *, gpointer );
-void on_main_menu_problem_example9    (GtkMenuItem *, gpointer );
+void on_main_menu_problem_example_n    (GtkMenuItem *, gpointer );
+void on_main_menu_preproc_noshift     (GtkMenuItem *, gpointer );
+void on_main_menu_preproc_shift     (GtkMenuItem *, gpointer );
+void on_main_menu_preproc_taushift     (GtkMenuItem *, gpointer );
+void on_main_menu_preproc_dbshift     (GtkMenuItem *, gpointer );
+void on_main_menu_preproc_autodual     (GtkMenuItem *, gpointer );
+void on_main_menu_preproc_ramadual     (GtkMenuItem *, gpointer );
+void on_main_menu_preproc_brigthdual     (GtkMenuItem *, gpointer );
+void on_main_menu_preproc_nodual     (GtkMenuItem *, gpointer );
+void on_main_menu_preproc_doubledual     (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_cyclicr_basic (GtkMenuItem *, gpointer );
-void on_main_menu_algorithms_cyclicr_shift (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_cyclicr_diag (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_logarithmicr_basic (GtkMenuItem *, gpointer );
-void on_main_menu_algorithms_logarithmicr_shift (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_logarithmicr_gth (GtkMenuItem *, gpointer );
-void on_main_menu_algorithms_funciter_basic0 (GtkMenuItem *, gpointer );
-void on_main_menu_algorithms_funciter_basic1 (GtkMenuItem *, gpointer );
-void on_main_menu_algorithms_funciter_basic_u (GtkMenuItem *, gpointer );
-void on_main_menu_algorithms_funciter_shift (GtkMenuItem *, gpointer );
-void on_main_menu_algorithms_funciter_shift_u (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_funciter_natural (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_funciter_traditional (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_funciter_Ubased (GtkMenuItem *, gpointer );
@@ -80,6 +70,12 @@ void on_main_menu_algorithms_funciter_readX(GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_invarsubsp_basic (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_invarsubsp_balzer (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_invarsubsp_shur (GtkMenuItem *, gpointer );
+void on_main_menu_algorithms_newton (GtkMenuItem *, gpointer );
+void on_main_menu_algorithms_funz_0 (GtkMenuItem *, gpointer );
+void on_main_menu_algorithms_funz_1 (GtkMenuItem *, gpointer );
+void on_main_menu_algorithms_funz_u (GtkMenuItem *, gpointer );
+void on_main_menu_algorithms_funz_readX(GtkMenuItem *, gpointer );
+
 void on_main_menu_goals_qbd_g (GtkMenuItem *, gpointer );
 void on_main_menu_goals_qbd_gr (GtkMenuItem *, gpointer );
 void on_main_menu_goals_qbd_gru (GtkMenuItem *, gpointer );
@@ -92,6 +88,7 @@ void on_main_menu_goals_pi_param (GtkMenuItem *, gpointer );
 void on_main_menu_options_verbose (GtkMenuItem *, gpointer );
 void on_main_menu_options_debug (GtkMenuItem *, gpointer );
 void on_main_menu_options_timings (GtkMenuItem *, gpointer );
+void on_main_menu_options_small (GtkMenuItem *, gpointer );
 void on_main_menu_options_situation (GtkMenuItem *, gpointer );
 void on_main_menu_options_eigth_d (GtkMenuItem *, gpointer );
 void on_main_menu_options_fifteen_d (GtkMenuItem *, gpointer );
@@ -101,6 +98,7 @@ void on_main_menu_options_parameters_cr (GtkMenuItem *, gpointer );
 void on_main_menu_options_parameters_lr (GtkMenuItem *, gpointer );
 void on_main_menu_options_parameters_fi (GtkMenuItem *, gpointer );
 void on_main_menu_options_parameters_is (GtkMenuItem *, gpointer );
+void on_main_menu_options_parameters_nt (GtkMenuItem *, gpointer );
 void on_main_menu_options_parameters_reset (GtkMenuItem *, gpointer );
 void on_examples_quit_activate    (GtkButton *, gpointer  );
 void on_examples_destroy_activate (GtkWidget  *ww, gpointer data);
@@ -137,6 +135,9 @@ void check_for_a_sparse(void);
 int examples_statusb_update(void);
 gint stop_computations(void);
 gboolean on_delete_event_do   (GtkWidget *, GdkEvent *, gpointer);
+void reset_dual(int);
+void menu_algorithm_keeper(void);
+void clean_algorithms_choice(void);
 
 /* prototypes help.c */
 void create_window_about(void);   
@@ -187,7 +188,7 @@ void create_view_window(void);
 /* prototypes view_pi.c */
 void create_view_pi_window(void);
 
-/* prototypes ponte.f90 */
+/* prototypes futiles.f90 */
 /* note: Fortran exports names in lower case only, so read_matA becomes read_mata_  ... */
 void getfdrift_(double *); 
 /* matrice A(k,i,j) */
@@ -223,16 +224,16 @@ void qbd_lr_solve_(int *,int *, int *,int *,int *);
 void qbd_fi_solve_(int *,int *, int *,int *,int *);
 void qbd_is_solve_(int *,int *, int *,int *,int *);
 void qbd_pi_ponte_(int *,int *, int *, int *, int *);
-void mg1_cr_solve_(int *,int *, int *,int *,int *,int *);
-void mg1_fi_solve_(int *,int *, int *,int *,int *);
-void mg1_is_solve_(int *,int *, int *,int *,int *);
+void mg1_cr_solve_(int *,int *, int *,int *,int *,int *,int *);
+void mg1_fi_solve_(int *,int *, int *,int *,int *,int *);
+void mg1_is_solve_(int *,int *, int *,int *,int *,int *);
 void mg1_pi_ponte_(int *,int *, int *, int *, int *);
-void gim1_cr_solve_(int *,int *, int *,int *,int *,int *);
-void gim1_lr_solve_(int *,int *, int *,int *,int *);
-void gim1_fi_solve_(int *,int *, int *,int *,int *);
-void gim1_is_solve_(int *,int *, int *,int *,int *);
+void gim1_cr_solve_(int *,int *, int *,int *,int *,int *,int *);
+void gim1_lr_solve_(int *,int *, int *,int *,int *,int *);
+void gim1_fi_solve_(int *,int *, int *,int *,int *,int *);
+void gim1_is_solve_(int *,int *, int *,int *,int *,int *);
 void gim1_pi_ponte_(int *,int *, int *, int *, int *);
-
+void ssdebug_(int *);
 
 /* prototypes examplesf.f90 */
 void examples_setup_(int *,int *,int *,int *,int *,int *,double *,double *);
@@ -247,6 +248,36 @@ int is_examples_to_wait; /* 0 no wait 1 wait 2 seconds before closing (error mes
 
 
 /* globals interface.c */
+GtkWidget *cyclicr_basic_m;
+
+GtkWidget *menu_qbd;
+GtkWidget *menu_mg1;
+GtkWidget *menu_gim1;
+
+GtkWidget *noshift_m;
+GtkWidget *simpleshift_m;
+GtkWidget *taushift_m;
+GtkWidget *doubleshift_m;
+GtkWidget *autodual_m;
+GtkWidget *brigthdual_m;
+GtkWidget *ramadual_m;
+GtkWidget *bothdual_m;
+GtkWidget *nodual_m;
+
+GtkWidget *cyclicr_basic_m;
+GtkWidget *cyclicr_diagadj_m;
+GtkWidget *logarithmicr_basic_m;
+GtkWidget *logarithmicr_gth_m;
+GtkWidget *funciter_natural_m;
+GtkWidget *funciter_traditional_m;
+GtkWidget *funciter_Ubased_m;
+GtkWidget *invarsubsp_basic_m;
+GtkWidget *invarsubsp_balzer_m;
+GtkWidget *invarsubsp_schur_m;
+GtkWidget *menuitem_newton;
+
+GtkWidget *funz_starting_m;
+
 GtkWidget *textview;
 GtkWidget *statusb;
 GtkWidget *window_examples;
@@ -289,7 +320,7 @@ double beta_in_mant;     /* used bye examples window */
 double beta_in_exp;      /* used bye examples window */
 double beta_in;          /* used bye examples window */
 char digits_to_save[64]; /* string corresponding to the number format chosen to write in files */
-struct parameters_str max[4],min[4],deflt[4],choice[4]; /* default parametres for each algorithm */
+struct parameters_str max[6],min[6],deflt[6],choice[6]; /* default parameters for each algorithm 1,2,3,4,5 */
 char param_interp_str[9][256], param_err_str[12][256]; /* actual strings used in combo */
 struct pi_parameters_str pi_max,pi_min,pi_deflt,pi_choice; /* default parametres for pi parameters setup */
 GtkWidget  *parameters_entry_maxnc,*parameters_entry_epspi;
@@ -300,6 +331,7 @@ struct running_problem_str committed, selected, completed; /* we save data here
 /* globals main.c */
 char my_host_name[512];
 char my_prog_name[512];
+int is_smallscr;
 
 /* globals examples.c */
 int  selected_es;  /* which example has been selected by menu in main window */
@@ -337,6 +369,9 @@ struct edita_str peb; /* preferences edit a */
 
 /* globals view_mat.c */
 struct edita_str pview; /* preferences view windows */
-
+GtkWidget * window_view;
+int is_view_fs_open;         /* 0 not opened, 1 opened */
+GtkWidget  *view_file_read;  /* window for choosing file to read */
+int view_file_activated; /* take note if button file has been selected */
 
 /****************************/
