@@ -6,7 +6,7 @@
  * Largo Pontecorvo 5                                         *
  * 56127 Pisa                                                 *
  * Italy                                                      *
- * Version 1.3 - March 2007                                   *
+ * Version 2.1 - June  2009                                   *
  **************************************************************
  *
  *  edit-b.h - view and edit B0, B, BN1  - both callbacks and interfaces
@@ -108,3 +108,7 @@ int is_edit_b_open;  /* 0=not open  1=open */
 int is_debug;    /* 1= debugging output active on console 0=normal */
 struct running_problem_str committed;
 struct edita_str peb; /* preferences edit a */
+
+/* globals main.c */
+int is_smallscr;
+
