@@ -6,7 +6,7 @@
  * Largo Pontecorvo 5                                         *
  * 56127 Pisa                                                 *
  * Italy                                                      *
- * Version 1.3 - March 2007                                   *
+ * Version 2.1 - June  2009                                   *
  **************************************************************
  *
  *  rw.h - reading and writing files
@@ -128,8 +128,11 @@ gint selection_uptodaten(void);
 /* prototypes callback.c */
 void invalidate_completed(void);
 gboolean on_delete_event_do   (GtkWidget *, GdkEvent *, gpointer);
+void reset_dual(int);
+void menu_algorithm_keeper(void);
+void clean_algorithms_choice(void);
 
-/* prototypes ponte.f90 */
+/* prototypes futils.f90 */
 /* note: Fortran exports names in lower case only, so read_matA becomes read_mata_  ... */
 /* matrice A(k,i,j) */
 void alloc_mata_(int *,int *, int *);
@@ -225,3 +228,5 @@ struct running_problem_str committed, selected, completed; /* we save data here
 
 /* globals edit_a.c */
 struct edita_str pea; /* preferences edit a */
+
+/*********************************************/
