@@ -6,7 +6,7 @@
  * Largo Pontecorvo 5                                         *
  * 56127 Pisa                                                 *
  * Italy                                                      *
- * Version 1.3 - March 2007                                   *
+ * Version 2.1 - June  2009                                   *
  **************************************************************
  *
  *  view_mat.c - view and compare G,R  - both callbacks and interfaces
@@ -41,8 +41,9 @@ create_view_window()
 /* GtkWidget *window_view; global */
 GtkWidget *vboxm_view;
 GtkWidget *hbox1,*hbox2;
-GtkWidget *vbox1,*vbox2;
+GtkWidget *vbox1,*vbox2,*vbox3;
 GtkWidget *vbox11,*vbox14;
+GtkWidget *hsep31;
 /* GtkWidget *vbox12,*vbox13; globals */
 GtkWidget *hsep11,*hsep12,*hsep13; /* separators */
 GtkWidget *label_v11,*label_v12,*label_v13; /* titles for button groups */
@@ -58,6 +59,7 @@ GtkWidget *button_range_block;
 GSList *range_list = NULL;
 GSList *matrix_list = NULL;
 GtkWidget *button_matrix_compare;
+GtkWidget *button_matrix_spectre;
 /* GtkWidget *button_matrix_g;   global to deselect when cancel */
 /* GtkWidget *button_matrix_r;   global to deselect when cancel */
 /* GtkWidget *button_matrix_u;    global to deselect when cancel */
@@ -66,8 +68,8 @@ GtkWidget *button_matrix_compare;
 GtkWidget *hsep1,*hsep2,*vsep0,*vsep1,*vsep2; /* separators */
 GtkWidget *vbox21,*vbox22, *vbox23;
 GtkWidget *label_211; /* labels with matrix results */
-GtkWidget *label_221; /* labels with matrix results */
 /* GtkWidget *label_v222, label_v212,*label_v231, *label_v232; globals */
+/* GtkWidget *label_v233; globals for spectre value */
 /* GtkWidget  *view_area; global */
 GtkWidget *hbox_matlabels;
 GtkWidget *label_mat,*label_vet;
@@ -77,7 +79,6 @@ GtkWidget *button_quit_view;
 GtkWidget *hbox_block;
 GtkWidget *hbox_aij;
 GtkWidget *label_aij;
-/* GtkWidget *view_label_space; global */
 /* GtkWidget   *view_entry_ij; global */
 /* GtkWidget   *view_entry_aij; global */
 GtkWidget *label_space2;
@@ -97,7 +98,9 @@ if(is_debug==1) printf("create_view_window: entering with completed.g=%d, comple
 /* window_view */
 window_view=gtk_window_new (GTK_WINDOW_TOPLEVEL);
 gtk_window_set_title (GTK_WINDOW (window_view), "View and Compare");
-gtk_window_set_default_size (GTK_WINDOW (window_view), 720, 700);
+if(is_smallscr==0) gtk_window_set_default_size (GTK_WINDOW (window_view), 720, 700);
+	else gtk_window_set_default_size (GTK_WINDOW (window_view), 510, 700);
+gtk_window_set_resizable (GTK_WINDOW (window_view),FALSE);
 gdk_rgb_init();
 
 /* vboxm_view - main vertical box*/
@@ -117,7 +120,7 @@ gtk_widget_show(vbox1);
 vbox11 = gtk_vbox_new(FALSE,5);
 gtk_box_pack_start (GTK_BOX (vbox1), vbox11, FALSE, FALSE, 0);
 gtk_widget_show(vbox11);
-label_v11 = gtk_label_new("DISPLAY CONTROL\n     change to: ");
+label_v11 = gtk_label_new("    DISPLAY CONTROL    \n       change to:    ");
 gtk_widget_show(label_v11);
 gtk_box_pack_start (GTK_BOX (vbox11), label_v11, TRUE, FALSE, 0);
 /* button positive / negative s_positive=0 default is positive i.e. black is 1 white is 0*/
@@ -252,15 +255,22 @@ switch (pview.is_gru) {
         case 3:
                 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button_matrix_u),TRUE);
                 break;
+	/* case 4 should not be possible at this point */
         }
 if (completed.g==0)  gtk_widget_set_sensitive(button_matrix_g,FALSE);
 if (completed.r==0)  gtk_widget_set_sensitive(button_matrix_r,FALSE);
 if (completed.u==0)  gtk_widget_set_sensitive(button_matrix_u,FALSE);
-/* Compare with File */
-button_matrix_compare = gtk_button_new_with_label("  Compare File  ");
-gtk_widget_show (button_matrix_compare);
-gtk_box_pack_start (GTK_BOX (vbox14), button_matrix_compare,TRUE, FALSE, 0);
-
+/* compute spectre */
+button_matrix_spectre = gtk_button_new_with_label(" Spectral Radius ");
+gtk_widget_show (button_matrix_spectre);
+label_v233 = gtk_label_new(" ");
+#if GTK_CHECK_VERSION(2,6,0)
+gtk_label_set_width_chars(GTK_LABEL(label_v233),22);
+#endif
+if(is_smallscr==0) {
+     gtk_box_pack_start (GTK_BOX (vbox14), button_matrix_spectre,TRUE, FALSE, 0);
+     gtk_box_pack_start (GTK_BOX (vbox14), label_v233,TRUE, FALSE, 0);
+	}
 /* vertical separator */
 vsep0 = gtk_vseparator_new();
 gtk_widget_show(vsep0);
@@ -269,7 +279,7 @@ gtk_box_pack_start (GTK_BOX (hbox1), vsep0, FALSE, FALSE, 0);
 /* second vbox in first hbox for matrix display */
 vbox2=gtk_vbox_new(FALSE,5);
 gtk_widget_show(vbox2);
-gtk_box_pack_end (GTK_BOX (hbox1), vbox2, FALSE, FALSE, 0);
+gtk_box_pack_start (GTK_BOX (hbox1), vbox2, FALSE, FALSE, 0);
 gtk_container_set_border_width (GTK_CONTAINER (vbox2), 10);
 /* matrix drawable area */
 gtk_widget_push_visual(gdk_rgb_get_visual());
@@ -297,66 +307,107 @@ gtk_box_pack_start (GTK_BOX (hbox_matlabels), view_label_user, FALSE, FALSE, 0);
 label_vet = gtk_label_new("Rows Sums");
 gtk_widget_show(label_vet);
 gtk_box_pack_end (GTK_BOX (hbox_matlabels), label_vet, FALSE, FALSE, 0);
+
 /* horizontal separator */
 hsep1 = gtk_hseparator_new();
 gtk_widget_show(hsep1);
 gtk_box_pack_start (GTK_BOX (vboxm_view), hsep1, FALSE, FALSE, 0);
 
+/* is small screen we use a third vbox on the right */
+if (is_smallscr==1) {
+vbox3=gtk_vbox_new(FALSE,5);
+gtk_box_pack_start (GTK_BOX (hbox1), vbox3, TRUE, TRUE, 0);
+gtk_widget_show(vbox3);
+}
+
 /* second hbox contains various data show boxes */
+/* in small screen they are in vbox3 */
 hbox2=gtk_hbox_new(FALSE,5);
 gtk_widget_show(hbox2);
-gtk_box_pack_start (GTK_BOX (vboxm_view), hbox2, FALSE, FALSE, 0);
+if (is_smallscr==0) gtk_box_pack_start (GTK_BOX (vboxm_view), hbox2, FALSE, FALSE, 0);
 /* matrix #blocks and dimensions */
 vbox21 = gtk_vbox_new(TRUE,5);
-gtk_box_pack_start (GTK_BOX (hbox2), vbox21, TRUE, TRUE, 0);
+if (is_smallscr==0) gtk_box_pack_start (GTK_BOX (hbox2), vbox21, TRUE, TRUE, 0);
+	else gtk_box_pack_start (GTK_BOX (vbox3), vbox21, TRUE, TRUE, 0);
 gtk_widget_show(vbox21);
 label_211 = gtk_label_new("   Matrix   ");
 gtk_widget_show(label_211);
-gtk_box_pack_start (GTK_BOX (vbox21), label_211, TRUE, TRUE, 0);
 label_v212 = gtk_label_new(" ");
 gtk_widget_show(label_v212);
 gtk_box_pack_start (GTK_BOX (vbox21), label_v212, FALSE, FALSE, 0);
+if (is_smallscr==0) {
 /* vertical separator */
 vsep1 = gtk_vseparator_new();
 gtk_widget_show(vsep1);
 gtk_box_pack_start (GTK_BOX (hbox2), vsep1, FALSE, FALSE, 0);
-vbox22 = gtk_vbox_new(TRUE,0);
-gtk_box_pack_start (GTK_BOX (hbox2), vbox22, TRUE, TRUE, 0);
-gtk_widget_show(vbox22);
-sprintf(buf,"Comparison");
-label_221 = gtk_label_new(buf);
-gtk_widget_show(label_221);
-gtk_box_pack_start (GTK_BOX (vbox22), label_221, TRUE, TRUE, 0);
-sprintf(buf,"    ");
-label_v222 = gtk_label_new(buf);
-gtk_widget_show(label_v222);
-gtk_box_pack_start (GTK_BOX (vbox22), label_v222, TRUE, TRUE, 0);
-/* vertical separator */
-vsep2 = gtk_vseparator_new();
-gtk_widget_show(vsep2);
-gtk_box_pack_start (GTK_BOX (hbox2), vsep2, FALSE, FALSE, 0);
+} else {
+/* horizontal separator */ 
+hsep31 = gtk_hseparator_new();
+gtk_widget_show(hsep31);
+gtk_box_pack_start (GTK_BOX (vbox3), hsep31, FALSE, FALSE, 0);
+}
+
+
 vbox23 = gtk_vbox_new(TRUE,0);
-gtk_box_pack_start (GTK_BOX (hbox2), vbox23, TRUE, TRUE, 0);
+if (is_smallscr==0) gtk_box_pack_start (GTK_BOX (hbox2), vbox23, TRUE, TRUE, 0);
+	else gtk_box_pack_start (GTK_BOX (vbox3), vbox23, TRUE, TRUE, 0);
 gtk_widget_show(vbox23);
-label_v231 = gtk_label_new(buf);
+label_v231 = gtk_label_new(" ");
 gtk_widget_show(label_v231);
 gtk_box_pack_start (GTK_BOX (vbox23), label_v231, TRUE, TRUE, 0);
-label_v232 = gtk_label_new(buf);
+label_v232 = gtk_label_new(" ");
 gtk_widget_show(label_v232);
 gtk_box_pack_start (GTK_BOX (vbox23), label_v232, TRUE, TRUE, 0);
 
+#if GTK_CHECK_VERSION(2,6,0)
+if (is_smallscr==0){
+	 gtk_label_set_width_chars(GTK_LABEL(label_v231),50);
+         gtk_label_set_width_chars(GTK_LABEL(label_v232),50);
+	}else {
+         gtk_label_set_width_chars(GTK_LABEL(label_v231),21);
+	 gtk_label_set_line_wrap (GTK_LABEL(label_v231),1);
+         gtk_label_set_width_chars(GTK_LABEL(label_v232),21);
+	gtk_label_set_line_wrap (GTK_LABEL(label_v232),1);
+	}
+#endif
+
 /* horizontal separator */
 hsep2 = gtk_hseparator_new();
 gtk_widget_show(hsep2);
-gtk_box_pack_start (GTK_BOX (vboxm_view), hsep2, FALSE, FALSE, 0);
+if (is_smallscr==0) gtk_box_pack_start (GTK_BOX (vboxm_view), hsep2, FALSE, FALSE, 0);
+	/* else gtk_box_pack_start (GTK_BOX (vbox3), hsep2, FALSE, FALSE, 0);*/
+if(is_smallscr==1) {
+     gtk_box_pack_start (GTK_BOX (vbox3), button_matrix_spectre,TRUE, FALSE, 0);
+     gtk_box_pack_start (GTK_BOX (vbox3), label_v233,TRUE, FALSE, 0);
+        }
+/*Compare with File button and results */
+vbox22 = gtk_vbox_new(TRUE,0);
+/* vertical separator */
+vsep2 = gtk_vseparator_new();
+gtk_widget_show(vsep2);
+if (is_smallscr==0)gtk_box_pack_start (GTK_BOX (hbox2), vsep2, FALSE, FALSE, 0);
+if (is_smallscr==0) gtk_box_pack_start (GTK_BOX (hbox2), vbox22, TRUE, TRUE, 0);
+        else gtk_box_pack_start (GTK_BOX (vbox3), vbox22, TRUE, TRUE, 0);
+gtk_widget_show(vbox22);
+button_matrix_compare = gtk_button_new_with_label(" Compare with File ");
+gtk_widget_show (button_matrix_compare);
+gtk_box_pack_start (GTK_BOX (vbox22), button_matrix_compare,TRUE, FALSE, 0);
+sprintf(buf,"   ");
+label_v222 = gtk_label_new(buf);
+gtk_widget_show(label_v222);
+gtk_box_pack_start (GTK_BOX (vbox22), label_v222, TRUE, FALSE, 0);
 
 /* third hbox consists in toolbox  */
 /* toolbar begins */
 toolbar_view = gtk_toolbar_new ();
+if (is_smallscr==1)
+gtk_toolbar_set_orientation(GTK_TOOLBAR (toolbar_view),GTK_ORIENTATION_VERTICAL);
 gtk_widget_show (toolbar_view);
-gtk_box_pack_start (GTK_BOX (vboxm_view), toolbar_view, FALSE, FALSE, 0);
-gtk_container_set_border_width (GTK_CONTAINER (toolbar_view), 10);
+if (is_smallscr==0) gtk_box_pack_start (GTK_BOX (vboxm_view), toolbar_view, FALSE, FALSE, 0);
+	else gtk_box_pack_start (GTK_BOX (vbox3), toolbar_view, FALSE, FALSE, 0);
+gtk_container_set_border_width (GTK_CONTAINER (toolbar_view), 5);
 gtk_toolbar_set_style (GTK_TOOLBAR (toolbar_view), GTK_TOOLBAR_BOTH);
+if (is_smallscr==0) {
 /* cancel button  first position */
 tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-quit", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar_view)));
 button_quit_view = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar_view),
@@ -365,15 +416,22 @@ button_quit_view = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar_view),
                                 "Close",
                                 NULL, NULL,
                                 tmp_toolbar_icon, NULL, NULL);
-gtk_widget_show(button_quit_view);
+ gtk_widget_show(button_quit_view);
+ }
 /* blocks choice hbox */
 hbox_block =gtk_hbox_new(TRUE,0);
 gtk_box_set_homogeneous (GTK_BOX(hbox_block),FALSE);
 
+if(is_smallscr==0){
 /* A(i,j)= x hbox */
 hbox_aij = gtk_hbox_new(TRUE,0);
+} else {
+/* A(i,j)= x vbox */
+hbox_aij = gtk_vbox_new(TRUE,0);
+}
 gtk_box_set_homogeneous (GTK_BOX(hbox_aij),FALSE);
-label_space2 = gtk_label_new("   Click to Memorize Value : ");
+if(is_smallscr==0) label_space2 = gtk_label_new("   Click to Memorize Value : ");
+	else label_space2 = gtk_label_new(" Click to Memorize Value : ");
 gtk_box_pack_start (GTK_BOX (hbox_aij), label_space2, FALSE, FALSE, 0);
 gtk_widget_show (label_space2);
 view_entry2_ij = gtk_entry_new_with_max_length (14);
@@ -403,18 +461,22 @@ gtk_box_pack_start (GTK_BOX (hbox_aij),view_entry_aij,FALSE, FALSE, 0);
 gtk_widget_show(gtk_toolbar_append_element (GTK_TOOLBAR (toolbar_view),
                                 GTK_TOOLBAR_CHILD_WIDGET,
                                 hbox_aij,
-                                "",
+                                " ",
                                 NULL, NULL,
                                 NULL, NULL, NULL));
-/* label space with editing/view only message */
-view_label_space = gtk_label_new(" ");
-gtk_widget_show (view_label_space);
-gtk_widget_show(gtk_toolbar_append_element (GTK_TOOLBAR (toolbar_view),
-                                GTK_TOOLBAR_CHILD_WIDGET,
-                                view_label_space,
-                                "",
+
+
+if (is_smallscr==1) {
+/* cancel button  last position */
+tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-quit", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar_view)));
+button_quit_view = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar_view),
+                                GTK_TOOLBAR_CHILD_BUTTON,
+                                NULL,
+                                "Close",
                                 NULL, NULL,
-                                NULL, NULL, NULL));
+                                tmp_toolbar_icon, NULL, NULL);
+ gtk_widget_show(button_quit_view);
+ }
 
 /* fourth hbox consists in statusbar  */
 /**************** statusbar  begins        */
@@ -463,6 +525,7 @@ g_signal_connect ((gpointer) button_matrix_u, "toggled", G_CALLBACK(on_view_u_ac
 g_signal_connect ((gpointer) button_matrix_file,"toggled", G_CALLBACK(on_view_file_activate),NULL);
 g_signal_connect ((gpointer) button_matrix_test,"toggled", G_CALLBACK(on_view_test_activate),NULL);
 g_signal_connect ((gpointer) button_matrix_compare,"clicked", G_CALLBACK(on_view_compare_activate),NULL);
+g_signal_connect ((gpointer) button_matrix_spectre,"clicked", G_CALLBACK(on_view_spectre_activate),NULL);
 /**************** connect signals  end     */
 
 gtk_widget_show_all(window_view);
@@ -584,12 +647,15 @@ if(pview.is_gru==3) {sprintf(buf,"U  %d x %d",completed.dimension_m,view_dimensi
 if((pview.is_gru==0)&&(view_is_file==1)) {sprintf(buf,"%s:  %d x %d",view_file_name,view_dimension,view_dimension);}
 if((pview.is_gru==0)&&(view_is_file==0)) {sprintf(buf,"Test Data 4 x 4");}
 gtk_label_set_text (GTK_LABEL(label_v212),buf);
-sprintf(buf,"max=%g  min=%g qmin=%g",view_mat_max,view_mat_min,view_mat_qmin);
+if(is_smallscr==0) sprintf(buf,"max=%g  min=%g qmin=%g",view_mat_max,view_mat_min,view_mat_qmin);
+	else  sprintf(buf," max=%g \n min=%g\n qmin=%g",view_mat_max,view_mat_min,view_mat_qmin);
 gtk_label_set_text (GTK_LABEL(label_v231),buf);
-sprintf(buf,"Sums:  Rows [%g ~ %g]   Col [%g ~ %g] ",vrmin,vrmax,vcmin,vcmax);
+if(is_smallscr==0) sprintf(buf,"Sums:  Rows [%g ~ %g]   Cols[%g ~ %g] ",vrmin,vrmax,vcmin,vcmax);
+	else sprintf(buf,"Sums:  Rows [%g~\n    ~ %g] \nSums:  Cols [%g~\n    ~ %g] ",vrmin,vrmax,vcmin,vcmax);
 gtk_label_set_text (GTK_LABEL(label_v232) ,buf);
-sprintf(buf,"Difference Norm=%g",view_compare_out);
+sprintf(buf,"DN=%g",view_compare_out);
 gtk_label_set_text (GTK_LABEL(label_v222),buf);
+gtk_label_set_text (GTK_LABEL(label_v233)," ");
 return(0);
 }
 /*********************************/
@@ -808,8 +874,11 @@ void
 on_view_file_activate(GtkButton *button, gpointer user_data)
 {
 if(is_debug==1) printf("on_view_file_activate: comparison=%d\n",is_view_comparison);
+if((pview.is_gru==0)&&(view_is_file==1)) return; /* do nothing when deselected  */
+
 if(view_file_activated==1) return; /* do not open file dialog on deselecting button */
 if(is_view_fs_open==1) {
+    gdk_beep();
     gtk_statusbar_push(GTK_STATUSBAR(statusb_view),0,"  You are just readin a file ! ");
     return;
     }
@@ -1059,7 +1128,8 @@ r=0.0;
 if(pview.is_gru==1) read_matg_(&i,&j,&r);
 if(pview.is_gru==2) read_matr_(&i,&j,&r);
 if(pview.is_gru==3) read_matu_(&i,&j,&r);
-if((pview.is_gru==0)&&(view_is_file==1)) r=view_file_data[i+view_dimension*(j-1)];
+/* old code if((pview.is_gru==0)&&(view_is_file==1)) r=view_file_data[i+view_dimension*(j-1)];*/
+if((pview.is_gru==0)&&(view_is_file==1)) read_matev_(&i,&j,&r);
 if(view_is_fake==1) r= fake_matrix[i+view_dimension*(j-1)];
 return(r);
 }
@@ -1468,8 +1538,14 @@ if((is_view_comparison==1)&&(ma!=view_dimension)) {
 		}
 /* if we are reading and not comparing, we must allocate the matrix */
 if(is_view_comparison==0) {
+/* old code - now ev is the matrix allocated by fortran code */
+/*
    view_file_data=(double *) malloc((ma+1)*(ma+1)*sizeof(double));
    if (view_file_data==NULL) {
+*/
+	err=0; /* returns 0 if no error */
+	alloc_matev_(&ma,&err);
+	if (err!=0) {	
         sprintf(buf,"\n  Allocation of A did Fail !\n(%ld Floats needed)\n",totfl);
         gtk_widget_show(create_dialog_data_noreadable(view_file_read,buf));
         view_fallback();
@@ -1498,7 +1574,8 @@ while(((c = getc(readfile)) != EOF)&&(err==0)) {
 						comp_diff=r-view_get_data(j,i);
 						view_compare_out=view_compare_out+comp_diff*comp_diff;
 						} else {
-                                                view_file_data[(i-1)*ma+j]=r; /* column j row i from 1 to ma fortran style */
+                                             /* view_file_data[(i-1)*ma+j]=r; column j row i from 1 to ma fortran style */
+						write_matev_(&j,&i,&r);
 						}
                         if(matlab_valid==0) {
                                 i++; 
@@ -1543,10 +1620,12 @@ view_is_file=1; /* 0=no 1=yes */
 view_is_fake=0;
 pview.is_gru=0;
 view_file_activated=1;
+
 free(vrs);
 free(vcs);
 vrs=malloc((view_dimension+1)*sizeof(double)); /* remember that index goes from 1 to dimension .. Fortran style */
 vcs=malloc((view_dimension+1)*sizeof(double)); /* remember that index goes from 1 to dimension .. Fortran style */
+
 max_min_view_compute();
 view_gray_levels_setup(pview.contrast_value);
 view_image_compute();
@@ -1563,7 +1642,16 @@ if(is_debug==1)printf("dim=%d name=%s view_is_file=%d\n",view_dimension,view_fil
 void 
 on_view_compare_activate(GtkButton *button, gpointer my_data)
 {
+/* nothing to compare if there is the test pattern ! */
+if(view_is_fake==1) {
+     gdk_beep();
+    gtk_statusbar_push(GTK_STATUSBAR(statusb_view),0,"  Compare with the test pattern not possible ! ");
+    return;
+    }
+
+
 if(is_view_fs_open==1) {
+     gdk_beep();
     gtk_statusbar_push(GTK_STATUSBAR(statusb_view),0,"  You are just readin a file ! ");
     return;
     }
@@ -1596,3 +1684,29 @@ if (view_is_fake==1) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button_matri
 view_file_activated=0;
 }
 /*********************************/
+
+/*
+ * compute spectral radius of G or R or U
+ *
+ */
+void
+on_view_spectre_activate(GtkButton *button, gpointer my_data)
+{
+char buf[256];
+double x;
+int s,k;
+x=-1;
+/* pview.is_gru: 0=no 1=g 2=r 3=u or view_is_file=1 */
+if(is_debug==1) printf("on_view_spectre_activate: pview.is_gru=%d view_is_file=%d\n",pview.is_gru,view_is_file);
+/* if pview.is_gru is 1, 2, 3 the matrix is G,R,U, if it is 0 and view_is_file is 1 the matrix is EV, else dummy data */
+if ((pview.is_gru==0)&&(view_is_file==0)) {
+		sprintf(buf,"???????????");
+		gtk_label_set_text (GTK_LABEL(label_v233),buf);
+	}	
+if ((pview.is_gru==0)&&(view_is_file==1)) k=4; else k=pview.is_gru;
+sprad_g_(&x,&s,&k);
+if(is_debug==1) printf("on_view_spectre_activate: Raggio spettrale = %20.16e matrice tipo %d (1=g 2=r 3=u 4=external values)\n",x,k);
+if( x < 0) sprintf(buf,"???????????"); else sprintf(buf,"%20.15e",x);
+gtk_label_set_text (GTK_LABEL(label_v233),buf);
+}
+/*********************************/
